# Generated by Django 3.2.7 on 2021-09-18 20:53

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("polaris", "0011_transaction_amount_expected"),
    ]

    operations = [
        migrations.AddField(
            model_name="transaction",
            name="account_memo",
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="transaction",
            name="muxed_account",
            field=models.TextField(blank=True, null=True),
        ),
    ]
