{% extends "./base.html" %} {% load i18n %}

{% block "title" %}
<title>{% if title %} {{ title|safe }} {% else %} {% blocktrans %}Error: {{ status_code }}{% endblocktrans %} {% endif %}</title>
{% endblock %}

{% block "content" %}
<section class="section">
    <h2>{% blocktrans %}Error: {{ status_code }}{% endblocktrans %}</h2>
    <p>{{ error }}</p>
</section>
{% endblock %}
