/* Box sizing rules */
/* Remove default padding */
ul[class],
ol[class] {
  padding: 0; }

/* Remove default margin */
body,
h1,
h2,
h3,
h4,
p,
ul[class],
ol[class],
li,
figure,
figcaption,
blockquote,
dl,
dd {
  margin: 0; }

/* Set core body defaults */
body {
  min-height: 100vh;
  scroll-behavior: smooth;
  text-rendering: optimizeSpeed;
  line-height: 1.5; }

/* Remove list styles on ul, ol elements with a class attribute */
ul[class],
ol[class] {
  list-style: none; }

a {
  color: inherit; }

/* A elements that don't have a class get default styles */
a:not([class]) {
  text-decoration-skip-ink: auto; }

/* Make images easier to work with */
img {
  max-width: 100%;
  display: block; }

/* Natural flow and rhythm in articles by default */
article > * + * {
  margin-top: 1em; }

/* Inherit fonts for inputs and buttons */
input,
button,
textarea,
select {
  font: inherit; }

/* Remove all animations and transitions for people that prefer not to see them */
@media (prefers-reduced-motion: reduce) {
  * {
    animation-duration: 0.01ms !important;
    animation-iteration-count: 1 !important;
    transition-duration: 0.01ms !important;
    scroll-behavior: auto !important; } }

/*! normalize.css v8.0.1 | MIT License | github.com/necolas/normalize.css */
/* Document
   ========================================================================== */
/**
 * 1. Correct the line height in all browsers.
 * 2. Prevent adjustments of font size after orientation changes in iOS.
 */
html {
  line-height: 1;
  /* 1 */
  -webkit-text-size-adjust: 100%;
  /* 2 */ }

/* Sections
   ========================================================================== */
/**
 * Remove the margin in all browsers.
 */
body {
  margin: 0; }

/**
 * Render the `main` element consistently in IE.
 */
main {
  display: block; }

/**
 * Correct the font size and margin on `h1` elements within `section` and
 * `article` contexts in Chrome, Firefox, and Safari.
 */
h1 {
  font-size: 2em;
  margin: 0.67em 0; }

/* Grouping content
   ========================================================================== */
/**
 * 1. Add the correct box sizing in Firefox.
 * 2. Show the overflow in Edge and IE.
 */
hr {
  box-sizing: content-box;
  /* 1 */
  height: 0;
  /* 1 */
  overflow: visible;
  /* 2 */ }

/**
 * 1. Correct the inheritance and scaling of font size in all browsers.
 * 2. Correct the odd `em` font sizing in all browsers.
 */
pre {
  font-family: monospace, monospace;
  /* 1 */
  font-size: 1em;
  /* 2 */ }

/* Text-level semantics
   ========================================================================== */
/**
 * Remove the gray background on active links in IE 10.
 */
a {
  background-color: transparent; }

/**
 * 1. Remove the bottom border in Chrome 57-
 * 2. Add the correct text decoration in Chrome, Edge, IE, Opera, and Safari.
 */
abbr[title] {
  border-bottom: none;
  /* 1 */
  text-decoration: underline;
  /* 2 */
  text-decoration: underline dotted;
  /* 2 */ }

/**
 * Add the correct font weight in Chrome, Edge, and Safari.
 */
b,
strong {
  font-weight: bolder; }

/**
 * 1. Correct the inheritance and scaling of font size in all browsers.
 * 2. Correct the odd `em` font sizing in all browsers.
 */
code,
kbd,
samp {
  font-family: monospace, monospace;
  /* 1 */
  font-size: 1em;
  /* 2 */ }

/**
 * Add the correct font size in all browsers.
 */
small {
  font-size: 80%; }

/**
 * Prevent `sub` and `sup` elements from affecting the line height in
 * all browsers.
 */
sub,
sup {
  font-size: 75%;
  line-height: 0;
  position: relative;
  vertical-align: baseline; }

sub {
  bottom: -0.25em; }

sup {
  top: -0.5em; }

/* Embedded content
   ========================================================================== */
/**
 * Remove the border on images inside links in IE 10.
 */
img {
  border-style: none; }

/* Forms
   ========================================================================== */
/**
 * 1. Change the font styles in all browsers.
 * 2. Remove the margin in Firefox and Safari.
 */
button,
input,
optgroup,
select,
textarea {
  font-family: inherit;
  /* 1 */
  font-size: 100%;
  /* 1 */
  line-height: 1.15;
  /* 1 */
  margin: 0;
  /* 2 */ }

/**
 * Show the overflow in IE.
 * 1. Show the overflow in Edge.
 */
button,
input {
  /* 1 */
  overflow: visible; }

/**
 * Remove the inheritance of text transform in Edge, Firefox, and IE.
 * 1. Remove the inheritance of text transform in Firefox.
 */
button,
select {
  /* 1 */
  text-transform: none; }

/**
 * Correct the inability to style clickable types in iOS and Safari.
 */
button,
[type="button"],
[type="reset"],
[type="submit"] {
  -webkit-appearance: button; }

/**
 * Remove the inner border and padding in Firefox.
 */
button::-moz-focus-inner,
[type="button"]::-moz-focus-inner,
[type="reset"]::-moz-focus-inner,
[type="submit"]::-moz-focus-inner {
  border-style: none;
  padding: 0; }

/**
 * Restore the focus styles unset by the previous rule.
 */
button:-moz-focusring,
[type="button"]:-moz-focusring,
[type="reset"]:-moz-focusring,
[type="submit"]:-moz-focusring {
  outline: 1px dotted ButtonText; }

/**
 * Correct the padding in Firefox.
 */
fieldset {
  padding: 0.35em 0.75em 0.625em; }

/**
 * 1. Correct the text wrapping in Edge and IE.
 * 2. Correct the color inheritance from `fieldset` elements in IE.
 * 3. Remove the padding so developers are not caught out when they zero out
 *    `fieldset` elements in all browsers.
 */
legend {
  box-sizing: border-box;
  /* 1 */
  color: inherit;
  /* 2 */
  display: table;
  /* 1 */
  max-width: 100%;
  /* 1 */
  padding: 0;
  /* 3 */
  white-space: normal;
  /* 1 */ }

/**
 * Add the correct vertical alignment in Chrome, Firefox, and Opera.
 */
progress {
  vertical-align: baseline; }

/**
 * Remove the default vertical scrollbar in IE 10+.
 */
textarea {
  overflow: auto; }

/**
 * 1. Add the correct box sizing in IE 10.
 * 2. Remove the padding in IE 10.
 */
[type="checkbox"],
[type="radio"] {
  box-sizing: border-box;
  /* 1 */
  padding: 0;
  /* 2 */ }

/**
 * Correct the cursor style of increment and decrement buttons in Chrome.
 */
[type="number"]::-webkit-inner-spin-button,
[type="number"]::-webkit-outer-spin-button {
  height: auto; }

/**
 * 1. Correct the odd appearance in Chrome and Safari.
 * 2. Correct the outline style in Safari.
 */
[type="search"] {
  -webkit-appearance: textfield;
  /* 1 */
  outline-offset: -2px;
  /* 2 */ }

/**
 * Remove the inner padding in Chrome and Safari on macOS.
 */
[type="search"]::-webkit-search-decoration {
  -webkit-appearance: none; }

/**
 * 1. Correct the inability to style clickable types in iOS and Safari.
 * 2. Change font properties to `inherit` in Safari.
 */
::-webkit-file-upload-button {
  -webkit-appearance: button;
  /* 1 */
  font: inherit;
  /* 2 */ }

/* Interactive
   ========================================================================== */
/*
 * Add the correct display in Edge, IE 10+, and Firefox.
 */
details {
  display: block; }

/*
 * Add the correct display in all browsers.
 */
summary {
  display: list-item; }

/* Misc
   ========================================================================== */
/**
 * Add the correct display in IE 10+.
 */
template {
  display: none; }

/**
 * Add the correct display in IE 10.
 */
[hidden] {
  display: none; }

.file-upload-wrapper {
  position: relative;
  width: 100%;
  height: 3rem; }
  .file-upload-wrapper:after {
    content: attr(data-text);
    position: absolute;
    top: 0;
    left: 0;
    background: rgba(7, 7, 11, 0.35);
    padding: 1rem 1.125rem;
    display: block;
    width: calc(100%);
    pointer-events: none;
    z-index: 20;
    height: 3rem;
    line-height: 1.15;
    border-radius: 0.75rem;
    font-weight: 300;
    cursor: pointer; }
  .file-upload-wrapper:hover:before {
    background: #0a8b74; }
  .file-upload-wrapper input {
    opacity: 0;
    padding: 1rem 1.125rem;
    z-index: 99;
    height: 1.15rem;
    margin: 0;
    display: block;
    cursor: pointer;
    width: 100%; }

.select-list__item {
  cursor: pointer;
  margin: 0.5rem 0 0.125rem 0;
  display: flex;
  flex-direction: row;
  align-items: center;
  transition: opacity 0.3s ease-out; }
  .select-list__item .select-list__body {
    display: flex;
    align-items: center;
    transition: all 0.15s ease-out;
    opacity: 0.4; }
  .select-list__item input[type="checkbox"] {
    -webkit-appearance: none;
    -moz-appearance: none;
    opacity: 0;
    display: block;
    width: 3rem;
    height: 3rem;
    background-repeat: no-repeat;
    background-position: left center; }
    .select-list__item input[type="checkbox"]:checked {
      opacity: 1;
      transform: scale(1); }
      .select-list__item input[type="checkbox"]:checked + .select-list__body {
        opacity: 1; }
    .select-list__item input[type="checkbox"] + .select-list__body:hover {
      opacity: 0.8; }
      .select-list__item input[type="checkbox"] + .select-list__body:hover .select-list__icon {
        transform: scale(1.05); }

.select-list__icon {
  transition: transform 0.3s ease-out; }

.select-list__content {
  margin-left: 2rem;
  display: flex;
  flex-direction: column; }

.select-list__title {
  font-size: 1.125rem;
  margin-bottom: 0.25rem; }

.select-list__description {
  font-size: 0.8125rem;
  line-height: 1.5;
  color: #94a0ac;
  font-weight: 500; }

.control input[type="checkbox"] {
  display: none; }
  .control input[type="checkbox"], .control input[type="checkbox"]:after, .control input[type="checkbox"]:before, .control input[type="checkbox"] *, .control input[type="checkbox"] *:after, .control input[type="checkbox"] *:before, .control input[type="checkbox"] + .tgl-btn {
    box-sizing: border-box; }
    .control input[type="checkbox"]::selection, .control input[type="checkbox"]:after::selection, .control input[type="checkbox"]:before::selection, .control input[type="checkbox"] *::selection, .control input[type="checkbox"] *:after::selection, .control input[type="checkbox"] *:before::selection, .control input[type="checkbox"] + .tgl-btn::selection {
      background: none; }
  .control input[type="checkbox"] + .tgl-btn {
    outline: 0;
    display: block;
    width: 4em;
    height: 2em;
    position: relative;
    cursor: pointer;
    user-select: none;
    background: rgba(148, 160, 172, 0.35);
    border-radius: 2em;
    padding: 2px;
    transition: all 0.4s ease; }
    .control input[type="checkbox"] + .tgl-btn:after {
      border-radius: 50%;
      background: #1a1b22;
      transition: all 0.2s ease; }
    .control input[type="checkbox"] + .tgl-btn:after, .control input[type="checkbox"] + .tgl-btn:before {
      position: relative;
      display: block;
      content: "";
      width: 50%;
      height: 100%; }
    .control input[type="checkbox"] + .tgl-btn:after {
      left: 0; }
    .control input[type="checkbox"] + .tgl-btn:before {
      display: none; }
  .control input[type="checkbox"]:checked + .tgl-btn {
    background: #0dbb9c; }
    .control input[type="checkbox"]:checked + .tgl-btn:after {
      left: 50%; }

.field {
  margin: 1rem 0; }

.label {
  display: block;
  font-size: 0.9375rem;
  color: #ffffff;
  padding: 0.25rem 0 0.5rem 0.25rem; }

.label .required {
  color: red; }

.button {
  display: block;
  cursor: pointer;
  background-color: #07070b;
  border: none;
  border-radius: 0.75rem;
  color: #ffffff;
  padding: 20px 30px;
  margin: 0 auto;
  width: 90%;
  max-width: 250px;
  transition: background-color 0.2s ease-out; }
  .button:focus {
    outline: none;
    box-shadow: 0 0 8px #0dbb9c, 0 0 1px #0dbb9c; }
  .button:hover {
    background-color: #11111b; }
  .button:disabled {
    background-color: #090909;
    opacity: 0.4; }
  .button:active {
    background-color: #07070b; }

::placeholder {
  color: #94a0ac; }

.radio-group {
  display: flex;
  flex-direction: column; }

.help {
  color: #e64755;
  padding: 0.25rem 0.5rem;
  font-size: 0.875rem; }

.help-text {
  color: grey;
  padding: 0.25rem 0.5rem;
  font-size: 0.875rem; }

.control {
  position: relative;
  cursor: pointer; }
  .control .icon {
    position: absolute;
    padding: 1rem;
    line-height: 1.125rem;
    font-weight: 700;
    border-radius: 0.75rem 0 0 0.75rem;
    background-color: #727b84; }
  .control input[type="text"],
  .control input[type="number"],
  .control input[type="date"],
  .control input[type="datetime-local"],
  .control input[type="time"],
  .control input[type="password"],
  .control input[type="tel"],
  .control input[type="email"] {
    width: 100%;
    background-color: rgba(7, 7, 11, 0.35);
    border-radius: 0.75rem;
    border: none;
    color: #ffffff;
    padding: 1rem 1rem;
    outline: none; }
    .control input[type="text"]:focus,
    .control input[type="number"]:focus,
    .control input[type="date"]:focus,
    .control input[type="datetime-local"]:focus,
    .control input[type="time"]:focus,
    .control input[type="password"]:focus,
    .control input[type="tel"]:focus,
    .control input[type="email"]:focus {
      box-shadow: 0 0 0.5rem rgba(51, 225, 255, 0.6), 0 0 0.125rem #33e1ff; }
    .control input[type="text"]:invalid:not(:placeholder-shown),
    .control input[type="number"]:invalid:not(:placeholder-shown),
    .control input[type="date"]:invalid:not(:placeholder-shown),
    .control input[type="datetime-local"]:invalid:not(:placeholder-shown),
    .control input[type="time"]:invalid:not(:placeholder-shown),
    .control input[type="password"]:invalid:not(:placeholder-shown),
    .control input[type="tel"]:invalid:not(:placeholder-shown),
    .control input[type="email"]:invalid:not(:placeholder-shown) {
      color: #e64755;
      box-shadow: 0 0 0.25rem rgba(230, 71, 85, 0.6), 0 0 0.125rem #e64755; }
    .control input[type="text"].polaris-transaction-form-amount,
    .control input[type="number"].polaris-transaction-form-amount,
    .control input[type="date"].polaris-transaction-form-amount,
    .control input[type="datetime-local"].polaris-transaction-form-amount,
    .control input[type="time"].polaris-transaction-form-amount,
    .control input[type="password"].polaris-transaction-form-amount,
    .control input[type="tel"].polaris-transaction-form-amount,
    .control input[type="email"].polaris-transaction-form-amount {
      padding-left: 2.25rem; }
  .control select,
  .control textarea {
    width: 100%;
    display: block;
    -moz-appearance: none;
    -webkit-appearance: none;
    appearance: none;
    outline: none;
    padding: 15px 40px 15px 18px;
    margin: 0;
    background-color: rgba(7, 7, 11, 0.35);
    border-radius: 0.75rem;
    border: none;
    color: #ffffff; }
    .control select:focus,
    .control textarea:focus {
      box-shadow: 0 0 8px rgba(51, 225, 255, 0.6), 0 0 1px #33e1ff; }
    .control select:invalid:not(:placeholder-shown),
    .control textarea:invalid:not(:placeholder-shown) {
      color: #94a0ac; }
  .control .radio-item {
    position: relative; }
    .control .radio-item label {
      font-size: 1.125rem;
      line-height: 2rem;
      margin: 5px 0;
      padding-left: 3rem;
      cursor: pointer;
      display: flex;
      align-items: center; }
  .control .radio-dot:before {
    content: "";
    display: block;
    position: absolute;
    left: 0;
    top: 0;
    background-color: rgba(7, 7, 11, 0.35);
    width: 2rem;
    height: 2rem;
    border-radius: 2rem; }
  .control .radio-dot:after {
    content: "";
    display: block;
    position: absolute;
    opacity: 0;
    transform: scale(0.8);
    transition: all 0.3s ease-out;
    background-color: #94a0ac;
    width: 1rem;
    height: 1rem;
    left: 0.5rem;
    top: 0.5rem;
    border-radius: 1rem; }
  .control input[type="radio"] {
    opacity: 0;
    width: 0px; }
    .control input[type="radio"]:checked + .radio-dot:after {
      opacity: 1;
      transform: scale(1); }

html {
  font-size: 16px;
  background-image: linear-gradient(to bottom, #303346, #1a1b22);
  color: #ffffff;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale; }

body {
  min-width: 15rem;
  box-sizing: border-box;
  font-family: "Cabin", sans-serif; }
  body *, body *:before, body *:after {
    box-sizing: inherit; }

.bottom {
  padding-bottom: 2rem;
  margin: auto; }

img.logo {
  width: 150px;
  height: 100px;
  margin-bottom: 10px;
  object-fit: contain; }

.main-content {
  overflow-x: hidden; }

.navbar {
  padding-top: 3rem;
  padding-bottom: 1rem;
  display: flex;
  justify-content: center;
  margin: auto; }

.navbar-brand {
  display: flex;
  flex-direction: column;
  align-items: center; }

.section {
  padding: 1rem;
  max-width: 25rem;
  margin: auto; }

.info-item--inline.info-item {
  padding: 0.5rem 0;
  display: flex;
  flex-direction: row;
  align-items: baseline; }

.info-item--inline .info-label {
  padding-right: 0.5rem;
  padding-bottom: 0; }

.info-item--inline .info-value {
  font-size: 0.875rem; }

.info-item {
  padding: 0.75rem 0; }
  .info-item.info-item--highlight {
    border-top: 0.0625rem solid rgba(255, 255, 255, 0.1);
    line-height: 1.5; }
    .info-item.info-item--highlight .info-label {
      color: #ffffff; }
    .info-item.info-item--highlight .info-value {
      color: #ffffff; }

.info-label {
  text-transform: capitalize;
  color: #94a0ac;
  font-size: 0.875rem;
  line-height: 1;
  padding-bottom: 0.25rem; }

.info-value {
  font-size: 1rem;
  color: #ffffff; }

.polaris-fee-table {
  width: 100%;
  border-collapse: collapse;
  line-height: 2;
  background-color: rgba(7, 7, 11, 0.35);
  border-radius: 0.75rem; }

.polaris-fee-row {
  border-bottom: 1px solid white; }

.polaris-fee-label {
  padding-left: 1rem; }

.polaris-fee-total-label {
  padding-left: 1rem; }

.polaris-fee-value {
  text-align: right;
  padding-right: 1rem; }

.polaris-fee-total-value {
  text-align: right;
  padding-right: 1rem; }
