# Generated by Django 2.2.16 on 2020-09-21 21:35

from django.db import migrations, models
import polaris.models


class Migration(migrations.Migration):
    dependencies = [
        ("polaris", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="transaction",
            name="envelope_xdr",
            field=models.TextField(
                null=True, validators=[polaris.models.deserialize], blank=True
            ),
        ),
        migrations.AddField(
            model_name="transaction",
            name="pending_signatures",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="transaction",
            name="channel_seed",
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="asset",
            name="distribution_account_signers",
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="asset",
            name="distribution_account_thresholds",
            field=models.TextField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name="asset",
            name="distribution_account_master_signer",
            field=models.TextField(null=True, blank=True),
        ),
    ]
