{% if widget.wrap_label %}
<label{% if widget.attrs.id %} for="{{ widget.attrs.id }}"{% endif %}>
{% endif %}
  {% include "django/forms/widgets/input.html" %}
  <div class="radio-dot"></div>
{% if widget.wrap_label %}
  {{ widget.label }}
</label>
{% endif %}
