# Generated by Django 2.2.19 on 2021-03-17 23:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("polaris", "0007_transaction_on_change_callback"),
    ]

    operations = [
        migrations.AddField(
            model_name="transaction",
            name="pending_execution_attempt",
            field=models.BooleanField(default=False),
        ),
    ]
